@@ -2,49 +2,48 @@
 
 """This is the core implementation of the evaluation."""
 import concurrent.futures
-import os
-import socket
 from threading import Lock, Thread
 
 import numpy as np
 import pandas as pd
 import torch
-from prometheus_client import Gauge, start_http_server
 from tensorboardX import SummaryWriter
-from tqdm import tqdm
+from tqdm.autonotebook import tqdm
 
 from ..utils import evaluation as eval_model
 from ..utils.common_util import print_dict_as_table, save_to_csv, timeit
-from ..utils.constants import DEFAULT_ITEM_COL, DEFAULT_PREDICTION_COL, DEFAULT_USER_COL
+from ..utils.constants import (
+    DEFAULT_ITEM_COL,
+    DEFAULT_PREDICTION_COL,
+    DEFAULT_RATING_COL,
+    DEFAULT_USER_COL,
+)
 from ..utils.seq_evaluation import mrr, ndcg, precision, recall
 
 lock_train_eval = Lock()
 lock_test_eval = Lock()
 
 
-def detect_port(port, ip="127.0.0.1"):
-    """Test whether the port is occupied.
+def computeRePos(time_seq, time_span):
+    """Compute position matrix for a user.
 
     Args:
-        port (int): port number.
-        ip (str): Ip address.
+        time_seq ([type]): [description]
+        time_span ([type]): [description]
 
     Returns:
-        True -- it's possible to listen on this port for TCP/IPv4 or TCP/IPv6
-                connections.
-        False -- otherwise.
+        [type]: [description]
     """
-    ready = True
-    try:
-        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        sock.bind((ip, port))
-        sock.listen(5)
-        sock.close()
-    except socket.error:
-        ready = False
-        raise RuntimeError("The server is already running on port {0}".format(port))
-    finally:
-        return ready
+    size = time_seq.shape[0]
+    time_matrix = np.zeros([size, size], dtype=np.int32)
+    for i in range(size):
+        for j in range(size):
+            span = abs(time_seq[i] - time_seq[j])
+            if span > time_span:
+                time_matrix[i][j] = time_span
+            else:
+                time_matrix[i][j] = span
+    return time_matrix
 
 
 def evaluate(data_df, predictions, metrics, k_li):
@@ -162,7 +161,9 @@ def test_eval_worker(testEngine, eval_data_df, prediction):
         eval_data_df, prediction, testEngine.metrics, testEngine.k
     )
     print_dict_as_table(
-        test_result_dic, tag="performance on test", columns=["metrics", "values"],
+        test_result_dic,
+        tag="performance on test",
+        columns=["metrics", "values"],
     )
     test_result_dic.update(result_para)
     lock_test_eval.acquire()  # need to be test
@@ -221,7 +222,6 @@ class EvalEngine(object):
         self.n_worker = 0
         self.n_no_update = 0
         self.best_valid_performance = 0
-        self.init_prometheus_env()
         print("Initializing test engine ...")
 
     def flush(self):
@@ -268,6 +268,222 @@ class EvalEngine(object):
             )
         return predictions
 
+    def seq_predict(self, train_seq, data_df, model, maxlen):
+        """Make prediction for a trained model.
+
+        Args:
+            data_df (DataFrame): A dataset to be evaluated.
+            model: A trained model.
+            batch_eval (Boolean): A signal to indicate if the model is evaluated in batches.
+
+        Returns:
+            array: predicted scores.
+        """
+        user_ids = data_df[DEFAULT_USER_COL].to_numpy()
+        item_ids = data_df[DEFAULT_ITEM_COL].to_numpy()
+        user_item_list = data_df.groupby([DEFAULT_USER_COL])[DEFAULT_ITEM_COL].apply(
+            list
+        )
+        result_dic = {}
+        with torch.no_grad():
+            for u, items in user_item_list.items():
+                if len(train_seq[u]) < 1:
+                    continue
+                seq = np.zeros([maxlen], dtype=np.int32)
+                idx = maxlen - 1
+                for i in reversed(train_seq[u]):
+                    seq[idx] = i
+                    idx -= 1
+                    if idx == -1:
+                        break
+                score = model.predict(
+                    np.array([u]),
+                    np.array([seq]),
+                    np.array(items),
+                )
+                score = np.array(
+                    score.flatten().to(torch.device("cpu")).detach().numpy() * -1
+                )
+                for i, item in enumerate(items):
+                    result_dic[(u, item)] = score[i]
+        predictions = []
+        for u, i in zip(user_ids, item_ids):
+            predictions.append(result_dic[(u, i)])
+        return np.array(predictions)
+
+    def test_seq_predict(self, train_seq, valid_data_df, test_data_df, model, maxlen):
+        """Make prediction for a trained model.
+
+        Args:
+            data_df (DataFrame): A dataset to be evaluated.
+            model: A trained model.
+            batch_eval (Boolean): A signal to indicate if the model is evaluated in batches.
+
+        Returns:
+            array: predicted scores.
+        """
+        valid_user_item_list = (
+            valid_data_df[valid_data_df[DEFAULT_RATING_COL] > 0]
+            .groupby([DEFAULT_USER_COL])[DEFAULT_ITEM_COL]
+            .apply(list)
+        )
+        user_item_list = test_data_df.groupby([DEFAULT_USER_COL])[
+            DEFAULT_ITEM_COL
+        ].apply(list)
+        user_ids = test_data_df[DEFAULT_USER_COL].to_numpy()
+        item_ids = test_data_df[DEFAULT_ITEM_COL].to_numpy()
+        predictions = []
+        result_dic = {}
+        with torch.no_grad():
+            for u, items in user_item_list.items():
+                if len(train_seq[u]) < 1:
+                    continue
+                seq = np.zeros([maxlen], dtype=np.int32)
+                idx = maxlen - 1
+                if u in valid_user_item_list:
+                    for i in reversed(
+                        valid_user_item_list[u]
+                    ):  # add validate item list
+                        seq[idx] = i
+                        idx -= 1
+                        if idx == -1:
+                            break
+                for i in reversed(train_seq[u]):
+                    if idx <= -1:
+                        break
+                    seq[idx] = i
+                    idx -= 1
+                    if idx == -1:
+                        break
+                score = model.predict(
+                    np.array([u]),
+                    np.array([seq]),
+                    np.array(items),
+                )
+                score = np.array(
+                    score.flatten().to(torch.device("cpu")).detach().numpy() * -1
+                )
+                for i, item in enumerate(items):
+                    result_dic[(u, item)] = score[i]
+        for u, i in zip(user_ids, item_ids):
+            predictions.append(result_dic[(u, i)])
+        return np.array(predictions)
+
+    # implemented this due to differences in indexing for TiSASRec, not sure if needed at all
+    def seq_predict_time(self, train_seq, data_df, model, maxlen, time_span):
+        """Make prediction for a trained model.
+
+        Args:
+            data_df (DataFrame): A dataset to be evaluated.
+            model: A trained model.
+            batch_eval (Boolean): A signal to indicate if the model is evaluated in batches.
+
+        Returns:
+            array: predicted scores.
+        """
+        user_ids = data_df[DEFAULT_USER_COL].to_numpy()
+        item_ids = data_df[DEFAULT_ITEM_COL].to_numpy()
+        user_item_list = data_df.groupby([DEFAULT_USER_COL])[DEFAULT_ITEM_COL].apply(
+            list
+        )
+        result_dic = {}
+        with torch.no_grad():
+            print("Train seq", type(train_seq))
+            for u, items in user_item_list.items():
+                if len(train_seq[u]) < 1:
+                    continue
+                seq = np.zeros([maxlen], dtype=np.int32)
+                time_seq = np.zeros([maxlen], dtype=np.int32)
+                time_matrix = computeRePos(time_seq, time_span)
+                idx = maxlen - 1
+                for i in reversed(train_seq[u]):
+                    seq[idx] = i[0]
+                    time_seq[idx] = i[1]
+                    idx -= 1
+                    if idx == -1:
+                        break
+                score = model.predict(
+                    np.array([u]),
+                    np.array([seq]),
+                    np.array([time_matrix]),
+                    np.array(items),
+                )
+                score = np.array(
+                    score.flatten().to(torch.device("cpu")).detach().numpy() * -1
+                )
+                for i, item in enumerate(items):
+                    result_dic[(u, item)] = score[i]
+        predictions = []
+        for u, i in zip(user_ids, item_ids):
+            predictions.append(result_dic[(u, i)])
+        return np.array(predictions)
+
+    # implemented this due to differences in indexing, not sure if needed
+    def test_seq_predict_time(
+        self, train_seq, valid_data_df, test_data_df, model, maxlen, time_span
+    ):
+        """Make prediction for a trained model.
+
+        Args:
+            data_df (DataFrame): A dataset to be evaluated.
+            model: A trained model.
+            batch_eval (Boolean): A signal to indicate if the model is evaluated in batches.
+
+        Returns:
+            array: predicted scores.
+        """
+        valid_user_item_list = (
+            valid_data_df[valid_data_df[DEFAULT_RATING_COL] > 0]
+            .groupby([DEFAULT_USER_COL])[DEFAULT_ITEM_COL]
+            .apply(list)
+        )
+        user_item_list = test_data_df.groupby([DEFAULT_USER_COL])[
+            DEFAULT_ITEM_COL
+        ].apply(list)
+        user_ids = test_data_df[DEFAULT_USER_COL].to_numpy()
+        item_ids = test_data_df[DEFAULT_ITEM_COL].to_numpy()
+        predictions = []
+        result_dic = {}
+        with torch.no_grad():
+            for u, items in user_item_list.items():
+                if len(train_seq[u]) < 1:
+                    continue
+                seq = np.zeros([maxlen], dtype=np.int32)
+                time_seq = np.zeros([maxlen], dtype=np.int32)
+                time_matrix = computeRePos(time_seq, time_span)
+                idx = maxlen - 1
+                if u in valid_user_item_list:
+                    for i in reversed(
+                        valid_user_item_list[u]
+                    ):  # add validate item list
+                        seq[idx] = i[0]
+                        time_seq[idx] = i[1]
+                        idx -= 1
+                        if idx == -1:
+                            break
+                for i in reversed(train_seq[u]):
+                    if idx <= -1:
+                        break
+                    seq[idx] = i[0]
+                    time_seq[idx] = i[1]
+                    idx -= 1
+                    if idx == -1:
+                        break
+                score = model.predict(
+                    np.array([u]),
+                    np.array([seq]),
+                    np.array([time_matrix]),
+                    np.array(items),
+                )
+                score = np.array(
+                    score.flatten().to(torch.device("cpu")).detach().numpy() * -1
+                )
+                for i, item in enumerate(items):
+                    result_dic[(u, item)] = score[i]
+        for u, i in zip(user_ids, item_ids):
+            predictions.append(result_dic[(u, i)])
+        return np.array(predictions)
+
     def train_eval(self, valid_data_df, test_data_df, model, epoch_id=0):
         """Evaluate the performance for a (validation) dataset with multiThread.
 
@@ -282,7 +498,82 @@ class EvalEngine(object):
         test_pred = self.predict(test_data_df, model, self.batch_eval)
         worker = Thread(
             target=train_eval_worker,
-            args=(self, valid_data_df, test_data_df, valid_pred, test_pred, epoch_id,),
+            args=(
+                self,
+                valid_data_df,
+                test_data_df,
+                valid_pred,
+                test_pred,
+                epoch_id,
+            ),
+        )
+        worker.start()
+
+    def seq_train_eval(
+        self, train_seq, valid_data_df, test_data_df, model, maxlen, epoch_id=0
+    ):
+        """Evaluate the performance for a (validation) dataset with multiThread.
+
+        Args:
+            valid_data_df (DataFrame): A validation dataset.
+            test_data_df (DataFrame): A testing dataset.
+            model: trained model.
+            epoch_id: epoch_id.
+            k (int or list): top k result to be evaluate.
+        """
+        valid_pred = self.seq_predict(train_seq, valid_data_df, model, maxlen)
+        test_pred = self.test_seq_predict(
+            train_seq, valid_data_df, test_data_df, model, maxlen
+        )
+        worker = Thread(
+            target=train_eval_worker,
+            args=(
+                self,
+                valid_data_df,
+                test_data_df,
+                valid_pred,
+                test_pred,
+                epoch_id,
+            ),
+        )
+        worker.start()
+
+    # implemented this due to differences in indexing, not sure if needed
+    def seq_train_eval_time(
+        self,
+        train_seq,
+        valid_data_df,
+        test_data_df,
+        model,
+        maxlen,
+        time_span,
+        epoch_id=0,
+    ):
+        """Evaluate the performance for a (validation) dataset with multiThread.
+
+        Args:
+            valid_data_df (DataFrame): A validation dataset.
+            test_data_df (DataFrame): A testing dataset.
+            model: trained model.
+            epoch_id: epoch_id.
+            k (int or list): top k result to be evaluate.
+        """
+        valid_pred = self.seq_predict_time(
+            train_seq, valid_data_df, model, maxlen, time_span
+        )
+        test_pred = self.test_seq_predict_time(
+            train_seq, valid_data_df, test_data_df, model, maxlen, time_span
+        )
+        worker = Thread(
+            target=train_eval_worker,
+            args=(
+                self,
+                valid_data_df,
+                test_data_df,
+                valid_pred,
+                test_pred,
+                epoch_id,
+            ),
         )
         worker.start()
 
@@ -325,72 +616,6 @@ class EvalEngine(object):
                 epoch_id,
             )
 
-    def init_prometheus_client(self):
-        """Initialize the prometheus http client."""
-        if "port" not in self.config["system"]:
-            port = 8003
-        else:
-            port = self.config["system"]["port"]
-        if detect_port(port):  # check if the port is available
-            print(f"port {port} is available. start_http_server.")
-            start_http_server(port)
-        else:
-            print(f"[Warning]: port {port} was already in use. ")
-            print(
-                "If you need to use prometheus, please check that port or specify"
-                " another port number."
-            )
-        gauges_test = {}
-        gauges_valid = {}
-        model_run_id = self.config["system"]["model_run_id"]
-        for metric in self.metrics:
-            gauges_test[metric] = Gauge(
-                metric + "_test" + f"_{model_run_id}",
-                "Model Testing Performance under " + metric,
-                self.tunable,
-            )
-            gauges_valid[metric] = Gauge(
-                metric + "_valid" + f"_{model_run_id}",
-                "Model Validation Performance under " + metric,
-                self.tunable,
-            )
-        self.gauges_test = gauges_test
-        self.gauges_valid = gauges_valid
-
-    def expose_performance(self, valid_result, test_result):
-        """Expose performance to a http_client.
-
-        Args:
-            valid_result (dict): Performance result of validation set.
-            test_result (dict): Performance result of testing set.
-        """
-        for metric in self.metrics:
-            self.gauges_valid[metric].labels(*self.labels).set(
-                valid_result[f"{metric}@{self.valid_k}"]
-            )
-            self.gauges_test[metric].labels(*self.labels).set(
-                test_result[f"{metric}@{self.valid_k}"]
-            )
-
-    def init_prometheus_env(self):
-        """Initialize prometheus environment."""
-        self.tunable = []
-        self.labels = []
-
-        for cfg in ["model", "dataset"]:
-            for col in self.config[cfg]["result_col"]:
-                self.tunable.append(col)
-                self.labels.append(self.config[cfg][col])
-
-        environs = ["objectID", "owner", "instance", "namespace", "appID"]
-        for environ in environs:
-            if environ in os.environ:
-                env_name = os.environ[environ]
-                self.tunable.append(environ)
-                self.labels.append(env_name)
-        self.labels = tuple(self.labels)
-        self.init_prometheus_client()
-
 
 class SeqEvalEngine(object):
     """The base evaluation engine for sequential recommendation."""
@@ -475,7 +700,14 @@ class SeqEvalEngine(object):
         return metrics / len(test_sequences)
 
     def evaluate_sequence(
-        self, recommender, seq, evaluation_functions, user, given_k, look_ahead, top_n,
+        self,
+        recommender,
+        seq,
+        evaluation_functions,
+        user,
+        given_k,
+        look_ahead,
+        top_n,
     ):
         """Compute metrics for each sequence.
 
@@ -550,7 +782,13 @@ class SeqEvalEngine(object):
         eval_cnt = 0
         for gk in range(given_k, len(seq), step):
             eval_res += self.evaluate_sequence(
-                recommender, seq, evaluation_functions, user, gk, look_ahead, top_n,
+                recommender,
+                seq,
+                evaluation_functions,
+                user,
+                gk,
+                look_ahead,
+                top_n,
             )
             eval_cnt += 1
         return eval_res / eval_cnt
