@@ -21,7 +21,8 @@ def tune_train(config):
     while train_engine.eval_engine.n_worker > 0:
         time.sleep(20)
     tune.report(
-        valid_metric=result["valid_metric"], model_save_dir=result["model_save_dir"],
+        valid_metric=result["valid_metric"],
+        model_save_dir=result["model_save_dir"],
     )
 
 
